unit Main;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, vgHasp;

type
  TMainForm = class(TForm)
    cmGetCodes: TButton;
    cmGetStatus: TButton;
    cmGetTime: TButton;
    cmSetTime: TButton;
    cmGetID: TButton;
    cmReadBlock: TButton;
    cmWriteBlock: TButton;
    cmGetDate: TButton;
    cmSetDate: TButton;
    cmReadByte: TButton;
    cmWriteByte: TButton;
    edRetCode1: TEdit;
    edSeedCode: TEdit;
    edPass2: TEdit;
    edPass1: TEdit;
    edIDNum: TEdit;
    edRetCode4: TEdit;
    edRetCode3: TEdit;
    edRetCode2: TEdit;
    edMemoryAddress: TEdit;
    edBlockLength: TEdit;
    edBuff: TEdit;
    edMemoryValue: TEdit;
    rgPort: TRadioGroup;
    cmLogin: TButton;
    cmLogout: TButton;
    cmReadWord: TButton;
    cmWriteWord: TButton;
    Hasp: TvgHasp;
    rgMode: TRadioGroup;
    edServer: TEdit;
    edCfg: TEdit;
    edProgID: TEdit;
    cmIdleTime: TButton;
    cmRefresh: TButton;
    cmReadBlockTimeHasp: TButton;
    cmWriteBlockTimeHasp: TButton;
    procedure rgModeClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure rgPortClick(Sender: TObject);
    procedure cmRefreshClick(Sender: TObject);
    procedure cmGetCodesClick(Sender: TObject);
    procedure cmGetStatusClick(Sender: TObject);
    procedure cmGetIDClick(Sender: TObject);
    procedure cmReadByteClick(Sender: TObject);
    procedure cmWriteByteClick(Sender: TObject);
    procedure cmReadWordClick(Sender: TObject);
    procedure cmWriteWordClick(Sender: TObject);
    procedure cmReadBlockClick(Sender: TObject);
    procedure cmWriteBlockClick(Sender: TObject);
    procedure cmGetTimeClick(Sender: TObject);
    procedure cmSetTimeClick(Sender: TObject);
    procedure cmGetDateClick(Sender: TObject);
    procedure cmSetDateClick(Sender: TObject);
    procedure cmLoginClick(Sender: TObject);
    procedure cmLogoutClick(Sender: TObject);
    procedure cmIdleTimeClick(Sender: TObject);
    procedure cmReadBlockTimeHaspClick(Sender: TObject);
    procedure cmWriteBlockTimeHaspClick(Sender: TObject);
  private
    procedure GetBuffer(var P: PChar; var Count: Integer);
    procedure SetBuffer(P: PChar; Count: Integer);
    procedure SetPasswords;
    procedure UpdateButtonStates;
    procedure UpdateParams(Params: TParams);
  public
    constructor Create(AOwner: TComponent); override;
  end;

var
  MainForm: TMainForm;

implementation
uses SysUtils;

{$R *.DFM}

procedure TMainForm.GetBuffer(var P: PChar; var Count: Integer);
var
  S, Tmp: String;
  I, J: Integer;
begin
  P := nil;
  S := edBuff.Text;
  Count := 0; P := nil;
  try
    Tmp := '';
    while S <> '' do
    begin
      if S[1] in ['0'..'9'] then Tmp := Tmp + S[1];

      if (Length(S) = 1) or not (S[1] in ['0'..'9']) then
      begin
        if Tmp <> '' then
        begin
          Inc(Count);
          J := StrToInt(Tmp);  Tmp := '';
          ReallocMem(P, Count);
          (P + Count - 1)^ := Char(J);
        end;
      end;

      Delete(S, 1, 1);
    end;
  except
    ReallocMem(P, 0);
    raise;
  end;
end;

procedure TMainForm.SetBuffer(P: PChar; Count: Integer);
var
  S: String;
  I: Integer;
  C: Char;
begin
  S := '';
  for I := 0 to Count - 1 do
  begin
    C := (P + I)^;
    if I > 0 then S := S + ' ';
    S := S + IntToStr(ord(C));
  end;
  edBuff.Text := S;
end;

procedure TMainForm.SetPasswords;
begin
  Hasp.Passwords[1] := StrToInt(edPass1.Text);
  Hasp.Passwords[2] := StrToInt(edPass2.Text);
end;

procedure TMainForm.UpdateButtonStates;
var
  IsHasp: Boolean;
begin
  IsHasp := Hasp.IsHasp;
  cmGetCodes.Enabled := IsHasp;
  cmGetStatus.Enabled := IsHasp;
  cmGetID.Enabled := IsHasp and (Hasp.Mode in [hmMemoHasp1, hmMemoHasp4, hmTimeHasp, hmTimeHasp4, hmNetHasp]);
  cmReadByte.Enabled := IsHasp and (Hasp.Mode in [hmTimeHasp, hmTimeHasp4]);
  cmWriteByte.Enabled := cmReadByte.Enabled;
  cmReadWord.Enabled := IsHasp and (Hasp.Mode in [hmMemoHasp1, hmMemoHasp4, hmTimeHasp, hmTimeHasp4, hmNetHasp]);
  cmWriteWord.Enabled := cmReadWord.Enabled;
  cmReadBlock.Enabled := IsHasp and (Hasp.Mode in [hmMemoHasp1, hmMemoHasp4, hmTimeHasp, hmTimeHasp4, hmNetHasp]);
  cmWriteBlock.Enabled := cmReadBlock.Enabled;
  cmReadBlockTimeHasp.Enabled := cmReadByte.Enabled;;
  cmWriteBlockTimeHasp.Enabled := cmReadByte.Enabled;
  cmGetTime.Enabled := cmReadByte.Enabled;
  cmSetTime.Enabled := cmReadByte.Enabled;
  cmGetDate.Enabled := cmReadByte.Enabled;
  cmSetDate.Enabled := cmReadByte.Enabled;
  cmLogin.Enabled := IsHasp and (Hasp.Mode in [hmNetHasp]) and not Hasp.Logged;
  cmLogout.Enabled := IsHasp and (Hasp.Mode in [hmNetHasp]) and Hasp.Logged;
  cmIdleTime.Enabled := cmLogout.Enabled;
end;

procedure TMainForm.UpdateParams(Params: TParams);
begin
  edRetCode1.Text := IntToStr(Params[1]);
  edRetCode2.Text := IntToStr(Params[2]);
  edRetCode3.Text := IntToStr(Params[3]);
  edRetCode4.Text := IntToStr(Params[4]);
end;

constructor TMainForm.Create(AOwner: TComponent);
begin
  RegisterClasses([TLabel]);
  inherited;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  rgPortClick(nil);
  rgModeClick(nil);
end;

procedure TMainForm.rgModeClick(Sender: TObject);
begin
  Hasp.Mode := THaspMode(rgMode.ItemIndex);
  cmRefreshClick(nil);
end;

procedure TMainForm.rgPortClick(Sender: TObject);
begin
  Hasp.Port := THaspPort(rgPort.ItemIndex);
end;

procedure TMainForm.cmRefreshClick(Sender: TObject);
begin
  UpdateButtonStates;
end;

procedure TMainForm.cmGetCodesClick(Sender: TObject);
var
  Params: TParams;
begin
  SetPasswords;
  Hasp.HaspCode(StrToInt(edSeedCode.Text), Params);
  UpdateParams(Params);
end;

procedure TMainForm.cmGetStatusClick(Sender: TObject);
var
  MemorySize: THaspMemorySize;
  HaspMode: THaspMode;
  HaspPort: Integer;
  Params: TParams;
begin
  SetPasswords;
  Hasp.HaspStatus(MemorySize, HaspMode, HaspPort);
  Params[1] := Integer(MemorySize);
  Params[2] := Integer(HaspMode);
  Params[3] := Integer(HaspPort);
  Params[4] := 0;
  UpdateParams(Params);
end;

procedure TMainForm.cmGetIDClick(Sender: TObject);
begin
  SetPasswords;
  edIDNum.Text := IntToStr(Hasp.HaspID);
end;

procedure TMainForm.cmReadByteClick(Sender: TObject);
begin
  SetPasswords;
  edMemoryValue.Text := IntToStr(Hasp.Bytes[StrToInt(edMemoryAddress.Text)]);
end;

procedure TMainForm.cmWriteByteClick(Sender: TObject);
begin
  SetPasswords;
  Hasp.Bytes[StrToInt(edMemoryAddress.Text)] := StrToInt(edMemoryValue.Text);
end;

procedure TMainForm.cmReadWordClick(Sender: TObject);
begin
  SetPasswords;
  edMemoryValue.Text := IntToStr(Hasp.Words[StrToInt(edMemoryAddress.Text)]);
end;

procedure TMainForm.cmWriteWordClick(Sender: TObject);
begin
  SetPasswords;
  Hasp.Words[StrToInt(edMemoryAddress.Text)] := StrToInt(edMemoryValue.Text);
end;

procedure TMainForm.cmReadBlockClick(Sender: TObject);
var
  P: PChar;
  I: Integer;
begin
  SetPasswords;
  I := StrToInt(edBlockLength.Text);
  GetMem(P, I * 2 + 1);
  try
    Hasp.CheckError(Hasp.CheckErrorCode, Hasp.ReadBlock(StrToInt(edMemoryAddress.Text), P^, I));
    SetBuffer(P, I * 2);
  finally
    FreeMem(P);
  end;
end;

procedure TMainForm.cmWriteBlockClick(Sender: TObject);
var
  P: PChar;
  I: Integer;
  Count: Integer;
begin
  SetPasswords;
  I := StrToInt(edBlockLength.Text);
  GetBuffer(P, Count);
  try
    if Count < I then I := Count;
    Hasp.CheckError(Hasp.CheckErrorCode, Hasp.WriteBlock(StrToInt(edMemoryAddress.Text), P^, I));
  finally
    FreeMem(P);
  end;
end;

procedure TMainForm.cmReadBlockTimeHaspClick(Sender: TObject);
var
  P: PChar;
  I: Integer;
begin
  SetPasswords;
  I := StrToInt(edBlockLength.Text);
  GetMem(P, I + 1);
  try
    Hasp.CheckError(Hasp.CheckErrorCode, Hasp.ReadBlockTimeHasp(StrToInt(edMemoryAddress.Text), P^, I));
    SetBuffer(P, I);
  finally
    FreeMem(P);
  end;
end;

procedure TMainForm.cmWriteBlockTimeHaspClick(Sender: TObject);
var
  P: PChar;
  I: Integer;
  Count: Integer;
begin
  SetPasswords;
  I := StrToInt(edBlockLength.Text);
  GetBuffer(P, Count);
  try
    if Count < I then I := Count;
    Hasp.CheckError(Hasp.CheckErrorCode, Hasp.WriteBlockTimeHasp(StrToInt(edMemoryAddress.Text), P^, I));
  finally
    FreeMem(P);
  end;
end;

procedure TMainForm.cmGetTimeClick(Sender: TObject);
var
  Time: TDateTime;
  Hours, Min, Sec, MSec: Word;
begin
  SetPasswords;
  Time := Hasp.Time;
  DecodeTime(Time, Hours, Min, Sec, MSec);
  edMemoryAddress.Text := IntToStr(Hours);
  edMemoryValue.Text := IntToStr(Min);
  edBlockLength.Text := IntToStr(Sec);
end;

procedure TMainForm.cmSetTimeClick(Sender: TObject);
var
  Time: TDateTime;
  Hours, Min, Sec, MSec: Word;
begin
  SetPasswords;
  Hours := StrToInt(edMemoryAddress.Text);
  Min := StrToInt(edMemoryValue.Text);
  Sec := StrToInt(edBlockLength.Text);
  MSec := 0;
  Hasp.Time := EncodeTime(Hours, Min, Sec, MSec);
end;

procedure TMainForm.cmGetDateClick(Sender: TObject);
var
  Date: TDateTime;
  Year, Month, Day: Word;
begin
  SetPasswords;
  Date := Hasp.Date;
  DecodeDate(Date, Year, Month, Day);
  edMemoryAddress.Text := IntToStr(Year);
  edMemoryValue.Text := IntToStr(Month);
  edBlockLength.Text := IntToStr(Day);
end;

procedure TMainForm.cmSetDateClick(Sender: TObject);
var
  Date: TDateTime;
  Year, Month, Day: Word;
begin
  SetPasswords;
  Year := StrToInt(edMemoryAddress.Text);
  Month := StrToInt(edMemoryValue.Text);
  Day := StrToInt(edBlockLength.Text);
  Date := EncodeDate(Year, Month, Day);
  Hasp.Date := Date;
end;

procedure TMainForm.cmLoginClick(Sender: TObject);
begin
  SetPasswords;
  Hasp.ServerName := edServer.Text;
  Hasp.ConfigFileName := edCfg.Text;
  Hasp.Logged := True;
  UpdateButtonStates;
end;

procedure TMainForm.cmLogoutClick(Sender: TObject);
begin
  SetPasswords;
  Hasp.Logged := False;
  UpdateButtonStates;
end;

procedure TMainForm.cmIdleTimeClick(Sender: TObject);
begin
  SetPasswords;
  Hasp.SetIdleTime(StrToInt(edSeedCode.Text));
end;

end.



